@@ -1,10 +1,33 @@
 use crate::array::*;
 use crate::numbers::*;
 
+use crate::arr;
+
 #[cfg(feature = "blas")]
 use crate::blas::{daxpy_blas, matmul_blas};
 
 impl Array {
+    /// Reshapes the array into different dimensions
+    pub fn reshape(&self, dimensions: Vec<usize>) -> Array {
+        let result = Array::from((Arc::new(dimensions), Arc::clone(&self.values)));
+
+        if !self.is_tracked {
+            result
+        } else {
+            let backward_op: BackwardOp = Arc::new(|c, t, x| {
+                vec![if t[0] {
+                    Some(x.reshape(c[0].dimensions()))
+                } else {
+                    None
+                }]
+            });
+
+            result
+                .with_backward_op(backward_op)
+                .with_children(vec![self.clone()])
+        }
+    }
+
     /// Computes the element-wise `alpha * x + y`, for each matching dimension not multiplied.
     pub fn axpy(alpha: Float, x: &Array, y: &Array) -> Array {
         #[cfg(not(feature = "blas"))]
@@ -13,11 +36,10 @@ impl Array {
         {
             let dimensions = element_wise_dimensions(&x.dimensions, &y.dimensions);
 
-            let op: SlicedOp =
-                Box::new(move |output_slice: &mut [Float], arrays: Vec<&[Float]>| {
-                    output_slice.clone_from_slice(&arrays[1]);
-                    daxpy_blas(alpha, arrays[0], output_slice);
-                });
+            let op: SlicedOp = Box::new(move |output_slice, arrays| {
+                output_slice.clone_from_slice(&arrays[1]);
+                daxpy_blas(alpha, arrays[0], output_slice);
+            });
 
             let backward_op: Option<BackwardOp> = if !x.is_tracked && !y.is_tracked {
                 None
@@ -87,7 +109,7 @@ impl Array {
                     sum += a[a_index] * b[b_index];
                 }
 
-                values[r * output_cols + j] = sum;
+                values[r * output_cols + j] += sum;
             }
         }
     }
@@ -98,7 +120,8 @@ impl Array {
     ///
     /// `a` - The LHS matrix, and whether to transpose it: `(a, a_transpose)`.
     /// `b` - The RHS matrix, and whether to transpose it: `(b, b_transpose)`.
-    fn matmul_values(a: (&Array, bool), b: (&Array, bool)) -> Array {
+    /// `c` - The output matrix, if not initialized to a zeros matrix.
+    pub fn matmul(a: (&Array, bool), b: (&Array, bool), c: Option<&Array>) -> Array {
         let (a, a_transpose) = a;
         let (b, b_transpose) = b;
 
@@ -158,7 +181,26 @@ impl Array {
             })
             .collect();
 
-        let op: SlicedOp = Box::new(move |output_slice: &mut [Float], arrays: Vec<&[Float]>| {
+        if let Some(c) = c {
+            if c.values.len() != 1 {
+                let is_cols_valid = c.dimensions[c.dimensions.len() - 1] == output_cols;
+                let is_rows_valid = c.dimensions.len() < 2
+                    || c.dimensions[c.dimensions.len() - 2] == 1
+                    || c.dimensions[c.dimensions.len() - 2] == output_rows;
+
+                if !is_cols_valid || !is_rows_valid {
+                    panic!("error: the dimensions {:?} cannot be broadcast to the output dimensions {:?}", c.dimensions, output_dimensions);
+                }
+            }
+        }
+
+        let op: SlicedOp = Box::new(move |output_slice, arrays| {
+            arrays[2]
+                .iter()
+                .cycle()
+                .take(output_slice.len())
+                .enumerate()
+                .for_each(|(i, x)| output_slice[i] = *x);
             #[cfg(feature = "blas")]
             matmul_blas(
                 (output_rows, output_cols, sum_len),
@@ -180,17 +222,17 @@ impl Array {
         } else {
             let backward_a = Box::new(move |c: &mut Vec<Array>, x: &Array| {
                 if a_transpose {
-                    Array::matmul_values((&c[1], b_transpose), (x, true))
+                    Array::matmul((&c[1], b_transpose), (x, true), None)
                 } else {
-                    Array::matmul_values((x, false), (&c[1], !b_transpose))
+                    Array::matmul((x, false), (&c[1], !b_transpose), None)
                 }
             });
 
             let backward_b = Box::new(move |c: &mut Vec<Array>, x: &Array| {
                 if b_transpose {
-                    Array::matmul_values((&x, true), (&c[0], a_transpose))
+                    Array::matmul((&x, true), (&c[0], a_transpose), None)
                 } else {
-                    Array::matmul_values((&c[0], !a_transpose), (&x, false))
+                    Array::matmul((&c[0], !a_transpose), (&x, false), None)
                 }
             });
 
@@ -198,28 +240,38 @@ impl Array {
                 vec![
                     if t[0] { Some(backward_a(c, x)) } else { None },
                     if t[1] { Some(backward_b(c, x)) } else { None },
+                    if t[2] { Some(x.clone()) } else { None },
                 ]
             }))
         };
 
-        Array::sliced_op(vec![a, b], &op, backward_op, &input_dimensions, &output_dimensions, 2, 0)
-    }
-
-    /// Computes matrix multiplications on two arrays, for each matching dimension not multiplied.
-    ///
-    /// # Arguments
-    ///
-    /// * `a` - The LHS matrix, and whether to transpose it: `(a, a_transpose)`.
-    /// * `b` - The RHS matrix, and whether to transpose it: `(b, b_transpose)`.
-    pub fn matmul(a: (&Array, bool), b: (&Array, bool)) -> Array {
-        Array::matmul_values(a, b)
+        let zeros = arr![0.0];
+        let c = if let Some(c) = c { c } else { &zeros };
+        Array::sliced_op(
+            vec![a, b, c],
+            &op,
+            backward_op,
+            &input_dimensions,
+            &output_dimensions,
+            2,
+            0,
+        )
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::arr;
+
+    #[test]
+    fn test_reshape() {
+        let a = arr![arr![1.0, 2.0], arr![3.0, 4.0]].tracked();
+        let mut reshaped = a.reshape(vec![1, 4]);
+        assert_eq!(reshaped, arr![arr![1.0, 2.0, 3.0, 4.0]]);
+
+        reshaped.backward(None);
+        assert_eq!(a.gradient().unwrap(), arr![arr![1.0, 1.0], arr![1.0, 1.0]]);
+    }
 
     #[test]
     fn test_axpy() {
@@ -246,7 +298,7 @@ mod tests {
 
         let matmul_expect = arr![arr![12.0, 21.0], arr![36.0, 54.0]];
 
-        let mut result = Array::matmul((&a, false), (&b, false));
+        let mut result = Array::matmul((&a, false), (&b, false), None);
         assert_eq!(result, matmul_expect);
 
         result.backward(None);
@@ -267,7 +319,7 @@ mod tests {
 
         let matmul_expect = arr![arr![12.0, 21.0], arr![36.0, 54.0]];
 
-        let mut result = Array::matmul((&a, true), (&b, false));
+        let mut result = Array::matmul((&a, true), (&b, false), None);
         assert_eq!(result, matmul_expect);
 
         result.backward(None);
@@ -290,7 +342,7 @@ mod tests {
         .tracked();
         let b = arr![arr![1.0, 2.0, 3.0]].tracked();
 
-        let mut result = Array::matmul((&a, false), (&b, true));
+        let mut result = Array::matmul((&a, false), (&b, true), None);
         assert_eq!(
             result,
             arr![arr![arr![14.0], arr![10.0]], arr![arr![32.0], arr![50.0]]]
@@ -312,7 +364,7 @@ mod tests {
         let a = arr![1.0, 2.0].tracked();
         let b = arr![arr![2.0, 4.0]].tracked();
 
-        let mut result = Array::matmul((&a, true), (&b, false));
+        let mut result = Array::matmul((&a, true), (&b, false), None);
         assert_eq!(result, arr![arr![2.0, 4.0], arr![4.0, 8.0]]);
 
         result.backward(None);
@@ -325,7 +377,7 @@ mod tests {
         let w = arr![arr![1.0, 2.0], arr![8.0, 2.0]].tracked();
         let x = arr![5.0, 3.0];
 
-        let mut result = Array::matmul((&x, false), (&w, true));
+        let mut result = Array::matmul((&x, false), (&w, true), None);
         result.backward(None);
         assert_eq!(w.gradient().unwrap(), arr![arr![5.0, 3.0], arr![5.0, 3.0]]);
     }
@@ -336,13 +388,13 @@ mod tests {
         let b = arr![arr![1.0, 2.0, 3.0]].tracked();
         let c = arr![arr![1.0], arr![2.0], arr![3.0]].tracked();
 
-        let result = Array::matmul((&a, false), (&b, true));
+        let result = Array::matmul((&a, false), (&b, true), None);
         assert_eq!(result, arr![arr![14.0], arr![32.0]]);
 
-        let result = Array::matmul((&b, false), (&a, true));
+        let result = Array::matmul((&b, false), (&a, true), None);
         assert_eq!(result, arr![arr![14.0, 32.0]]);
 
-        let result = Array::matmul((&b, false), (&c, false));
+        let result = Array::matmul((&b, false), (&c, false), None);
         assert_eq!(result, arr![arr![14.0]]);
     }
 
@@ -350,7 +402,7 @@ mod tests {
     fn test_matmul_single() {
         let a = arr![1.0, 2.0, 3.0];
         let b = arr![3.0, 2.0, 1.0];
-        let result = Array::matmul((&a, false), (&b, false));
+        let result = Array::matmul((&a, false), (&b, false), None);
         assert_eq!(result, arr![10.0]);
     }
 
@@ -361,8 +413,9 @@ mod tests {
         let c = arr![arr![1.0, 2.0, 3.0]];
 
         let result = Array::matmul(
-            (&Array::matmul((&a, false), (&b, false)), false),
+            (&Array::matmul((&a, false), (&b, false), None), false),
             (&c, false),
+            None,
         );
         assert_eq!(result, arr![arr![14.0, 28.0, 42.0], arr![32.0, 64.0, 96.0]]);
     }
@@ -402,7 +455,7 @@ mod tests {
             ]
         ];
 
-        let result = Array::matmul((&a, false), (&b, false));
+        let result = Array::matmul((&a, false), (&b, false), None);
         assert_eq!(result, matmul_expect);
     }
 
@@ -411,7 +464,7 @@ mod tests {
         let a = arr![arr![1.0, 2.0, 3.0]].tracked();
         let b = arr![arr![9.0, 8.0, 7.0]].tracked();
 
-        let mut result = Array::matmul((&a, false), (&b, true));
+        let mut result = Array::matmul((&a, false), (&b, true), None);
 
         result.backward(None);
         assert_eq!(a.gradient().unwrap(), arr![arr![9.0, 8.0, 7.0]]);
@@ -424,7 +477,8 @@ mod tests {
         let b = arr![arr![1.0], arr![2.0], arr![3.0]].tracked();
         let c = arr![arr![7.0], arr![8.0]].tracked();
 
-        let mut result = &Array::matmul((&a, false), (&b, false)) + &c;
+        let mut result = Array::matmul((&a, false), (&b, false), Some(&c));
+        assert_eq!(result, arr![arr![21.0], arr![40.0]]);
 
         result.backward(None);
         assert_eq!(c.gradient().unwrap(), arr![arr![1.0], arr![1.0]]);
@@ -442,8 +496,9 @@ mod tests {
         let c = arr![arr![1.0, 2.0, 3.0]].tracked();
 
         let mut result = Array::matmul(
-            (&Array::matmul((&a, false), (&b, false)), false),
+            (&Array::matmul((&a, false), (&b, false), None), false),
             (&c, false),
+            None,
         );
 
         result.backward(None);
