@@ -12,14 +12,14 @@ extern crate approx;
 pub mod numbers;
 #[macro_use]
 pub mod array;
+pub mod activation;
 #[cfg(feature = "blas")]
 pub mod blas;
+pub mod cost;
+pub mod initializer;
 pub mod layer;
-pub mod layers;
 pub mod model;
-pub mod nn;
 pub mod optimizer;
-pub mod optimizers;
 
 #[cfg(test)]
 mod tests {
